use crate::patterns::fade::Fade;
use crate::Strip;
use serde::{Serialize, Deserialize};
use crate::patterns::Pattern;

/// # Blink
///
/// fade from one color to an other then go back several times
#[derive(Serialize, Deserialize, Debug, Eq, Hash, PartialEq, Copy, Clone)]
pub struct Blink<const N: usize> {
    #[serde(default)]
    pub(crate) current_iteration: usize,
    pub(crate) max_iteration: Option<usize>,
    fade: Fade<N>,
}

impl<const N: usize> Pattern for Blink<N> {
    type Strip = Strip<N>;

    fn init(&self) -> Option<Self::Strip> {
        None
    }
}

impl<const N: usize> Iterator for Blink<N> {
    type Item = Strip<N>;

    fn next(&mut self) -> Option<Self::Item> {
        let value = self.fade.next();
        match value {
            None => {
                if let Some(iter_max) = self.max_iteration {
                    if self.current_iteration >= iter_max {
                        return None;
                    }
                    self.current_iteration += 1;
                }
                self.fade.current_iteration = 0;
                std::mem::swap(&mut self.fade.begin_color, &mut self.fade.end_color);
                self.fade.next()
            }
            Some(value) => Some(value),
        }
    }
}