use rand::Rng;
use crate::patterns::{Pattern, PixelColor};
use crate::Strip;
use serde::{Serialize, Deserialize};

/// # FillUnstable
///
/// fill strip with color then apply same random variation to all pixel
#[derive(Serialize, Deserialize, Debug, Eq, Hash, PartialEq, Copy, Clone)]
pub struct FillUnstable<const N: usize> {
    pub(crate) color: PixelColor,
    pub(crate) stability: usize,
    pub(crate) max_iteration: Option<usize>,
}

impl<const N: usize> Pattern for FillUnstable<N> {
    type Strip = Strip<N>;

    fn init(&self) -> Option<Self::Strip> {
        None
    }
}

impl<const N: usize> Iterator for FillUnstable<N> {
    type Item = Strip<N>;

    fn next(&mut self) -> Option<Self::Item> {
        if let Some(iteration) = self.max_iteration {
            if iteration == 0 {
                return None;
            } else {
                self.max_iteration = Some(iteration - 1);
            }
        }

        let mut strip = Strip::<N>::default();
        let mut rng = rand::thread_rng();
        let red_delta = rng.gen_range(0..self.stability) as u8;
        let green_delta = rng.gen_range(0..self.stability) as u8;
        let blue_delta = rng.gen_range(0..self.stability) as u8;
        let operation = rng.gen_bool(0.5);
        let red;
        let green;
        let blue;
        if operation {
            red = self.color.red.saturating_add(red_delta);
            green = self.color.green.saturating_add(green_delta);
            blue = self.color.blue.saturating_add(blue_delta);
        } else {
            red = self.color.red.saturating_sub(red_delta);
            green = self.color.green.saturating_sub(green_delta);
            blue = self.color.blue.saturating_sub(blue_delta);
        }
        strip.fill(PixelColor { red, green, blue });
        Some(strip)
    }
}